#include "RooGlobalFunc.h"
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "RooPolyVar.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "RooMCStudy.h"
#include "TFile.h"
#include "RooJohnson.h"
#include "TRandom3.h"
#include "RooBinning.h"
#include "RooDataHist.h"
#include "RooExponential.h"
#include "RooChebychev.h"
#include "RooDstD0BG.h"
#include "myRooPolBG.cpp"
#include "myRooJohnsonSU.cpp"
#include <fstream>
#include <iostream>
#include <sstream>
// #include <ofstream>
#include <Riostream.h>


using namespace RooFit ;
using namespace std;


void generation(int i_toy) 
{

  //The Variables
  RooRealVar mD0pi("mD0pi","", 2.005, 2.025, "");
  RooRealVar mKK("mKK","", 1.75, 2.05, "");

  /*-----------------------S I G N A L----------------------*/

  //d0pi
  RooRealVar f_J_sig_d0pi("f_{J_sig_d0pi}", "rel_frac_j",4.39599e-01);
  RooRealVar A_fJ_sig_d0pi("A_{fJ_sig_d0pi}","",0);
  RooFormulaVar f_J_sig_d0pi_d0("f_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(f_J_sig_d0pi,A_fJ_sig_d0pi));
  RooFormulaVar f_J_sig_d0pi_d0bar("f_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(f_J_sig_d0pi,A_fJ_sig_d0pi));

  RooRealVar mu_J_sig_d0pi("#mu_{J_sig_d0pi}","mean_johnson",2.01024e+00,2.009,2.011);
  RooRealVar A_mu_J_sig_d0pi("A_{mu_J_sig_d0pi}","",0);
  RooFormulaVar mu_J_sig_d0pi_d0("mu_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(mu_J_sig_d0pi, A_mu_J_sig_d0pi));
  RooFormulaVar mu_J_sig_d0pi_d0bar("mu_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(mu_J_sig_d0pi, A_mu_J_sig_d0pi));

  RooRealVar sigma_J_sig_d0pi("#sigma_{J_sig_d0pi}", "sigma_johnson",2.35832e-04);
  RooRealVar A_sigma_J_sig_d0pi("A_{sigma_J_sig_d0pi}","",0);
  RooFormulaVar sigma_J_sig_d0pi_d0("sigma_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_J_sig_d0pi, A_sigma_J_sig_d0pi));
  RooFormulaVar sigma_J_sig_d0pi_d0bar("sigma_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_sig_d0pi, A_sigma_J_sig_d0pi));

  RooRealVar gamma_J_sig_d0pi("#gamma_{J_sig_d0pi}", "gamma",-2.02236e-01);
  RooRealVar A_gamma_J_sig_d0pi("A_{gamma_J_sig_d0pi}","",0);
  RooFormulaVar gamma_J_sig_d0pi_d0("gamma_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(gamma_J_sig_d0pi, A_gamma_J_sig_d0pi));
  RooFormulaVar gamma_J_sig_d0pi_d0bar("gamma_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_sig_d0pi, A_gamma_J_sig_d0pi));

  RooRealVar delta_J_sig_d0pi("#delta_{J_sig_d0pi}", "delta", 9.49540e-01);
  RooRealVar A_delta_J_sig_d0pi("A_{delta_J_sig_d0pi}","",0);
  RooFormulaVar delta_J_sig_d0pi_d0("delta_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(delta_J_sig_d0pi, A_delta_J_sig_d0pi));
  RooFormulaVar delta_J_sig_d0pi_d0bar("delta_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(delta_J_sig_d0pi, A_delta_J_sig_d0pi));

  myRooJohnsonSU johnson_sig_d0pi_d0("johnson_sig_d0pi_d0","johnson_sig_d0", mD0pi, mu_J_sig_d0pi_d0, sigma_J_sig_d0pi_d0, delta_J_sig_d0pi_d0, gamma_J_sig_d0pi_d0);
  myRooJohnsonSU johnson_sig_d0pi_d0bar("johnson_sig_d0pi_d0bar","johnson_sig_d0bar", mD0pi, mu_J_sig_d0pi_d0bar, sigma_J_sig_d0pi_d0bar, delta_J_sig_d0pi_d0bar, gamma_J_sig_d0pi_d0bar);

  RooRealVar f_G1_sig_d0pi("f_{G1sig_d0pi}", "frac_gauss1",3.26131e-01);
  RooRealVar A_f_G1_sig_d0pi("A_{f_G1_sig_d0pi}","",0);
  RooFormulaVar f_G1_sig_d0pi_d0("f_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(f_G1_sig_d0pi, A_f_G1_sig_d0pi));
  RooFormulaVar f_G1_sig_d0pi_d0bar("f_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(f_G1_sig_d0pi, A_f_G1_sig_d0pi));

  RooRealVar mu_G1_sig_d0pi("#mu_{G1sig_d0pi}", "mean_gauss1",2.01025e+00);
  RooRealVar A_mu_G1_sig_d0pi("A_{mu_G1_sig_d0pi}","",0);
  RooFormulaVar mu_G1_sig_d0pi_d0("mu_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(mu_G1_sig_d0pi, A_mu_G1_sig_d0pi));
  RooFormulaVar mu_G1_sig_d0pi_d0bar("mu_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(mu_G1_sig_d0pi, A_mu_G1_sig_d0pi));

  RooRealVar sigma_G1_sig_d0pi("#sigma_{G1sig_d0pi}", "sigma_gauss1",3.76484e-04);
  RooRealVar A_sigma_G1_sig_d0pi("A_{sigma_G1_sig_d0pi}","",0);
  RooFormulaVar sigma_G1_sig_d0pi_d0("sigma_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_G1_sig_d0pi, A_sigma_G1_sig_d0pi));
  RooFormulaVar sigma_G1_sig_d0pi_d0bar("sigma_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G1_sig_d0pi, A_sigma_G1_sig_d0pi));

  RooRealVar sigma_G2_sig_d0pi("#sigma_{G2sig_d0pi}", "sigma_gauss2",2.21655e-04);
  RooRealVar A_sigma_G2_sig_d0pi("A_{sigma_G2_sig_d0pi}","",0);
  RooFormulaVar sigma_G2_sig_d0pi_d0("sigma_G2_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_G2_sig_d0pi, A_sigma_G2_sig_d0pi));
  RooFormulaVar sigma_G2_sig_d0pi_d0bar("sigma_G2_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G2_sig_d0pi, A_sigma_G2_sig_d0pi));

  RooGaussian gauss1_sig_d0pi_d0("gauss1_sig_d0pi_d0", "1st gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0, sigma_G1_sig_d0pi_d0);
  RooGaussian gauss1_sig_d0pi_d0bar("gauss1_sig_d0pi_d0bar", "1st gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0bar, sigma_G1_sig_d0pi_d0bar);
  RooGaussian gauss2_sig_d0pi_d0("gauss2_sig_d0pi_d0", "2nd gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0, sigma_G2_sig_d0pi_d0);
  RooGaussian gauss2_sig_d0pi_d0bar("gauss2_sig_d0pi_d0bar", "2nd gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0bar, sigma_G2_sig_d0pi_d0bar);

  RooAddPdf doublegaussian_sig_d0pi_d0("doublegaussian_sig_d0", "doublegaussian_sig", gauss1_sig_d0pi_d0, gauss2_sig_d0pi_d0, f_G1_sig_d0pi_d0);
  RooAddPdf doublegaussian_sig_d0pi_d0bar("doublegaussian_sig_d0bar", "doublegaussian_sig", gauss1_sig_d0pi_d0bar, gauss2_sig_d0pi_d0bar, f_G1_sig_d0pi_d0bar);

  RooAddPdf sig_mD0pi_d0("sig_mD0pi_d0","sig_mD0pi_d0", johnson_sig_d0pi_d0, doublegaussian_sig_d0pi_d0, f_J_sig_d0pi_d0);
  RooAddPdf sig_mD0pi_d0bar("sig_mD0pi_d0bar","sig_mD0pi_d0bar", johnson_sig_d0pi_d0bar, doublegaussian_sig_d0pi_d0bar, f_J_sig_d0pi_d0bar);

  //kk
  RooRealVar f_J_sig_dz("f_{J_sig_dz}", "relative_frac_j",9.80741e-01);
  RooRealVar A_f_J_sig_dz("A_{f_J_sig_dz}","",0);
  RooFormulaVar f_J_sig_dz_d0("f_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(f_J_sig_dz, A_f_J_sig_dz));
  RooFormulaVar f_J_sig_dz_d0bar("f_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(f_J_sig_dz, A_f_J_sig_dz));

  RooRealVar mu_J_sig_dz("#mu_{J_sig_dz}","mean_johnson",1.86436e+00,1.85,1.87);
  RooRealVar A_mu_J_sig_dz("A_{mu_J_sig_dz}","",0);
  RooFormulaVar mu_J_sig_dz_d0("mu_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(mu_J_sig_dz, A_mu_J_sig_dz));
  RooFormulaVar mu_J_sig_dz_d0bar("mu_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(mu_J_sig_dz, A_mu_J_sig_dz));

  RooRealVar sigma_J_sig_dz("#sigma_{J_sig_dz}", "sigma_johnson",9.51422e-03,0,1);
  RooRealVar A_sigma_J_sig_dz("A_{sigma_J_sig_dz}","",0);
  RooFormulaVar sigma_J_sig_dz_d0("sigma_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_J_sig_dz, A_sigma_J_sig_dz));
  RooFormulaVar sigma_J_sig_dz_d0bar("sigma_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_sig_dz, A_sigma_J_sig_dz));

  RooRealVar gamma_J_sig_dz("#gamma_{J_sig_dz}", "gamma",-1.24780e-01);
  RooRealVar A_gamma_J_sig_dz("A_{gamma_J_sig_dz}","",0);
  RooFormulaVar gamma_J_sig_dz_d0("gamma_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(gamma_J_sig_dz, A_gamma_J_sig_dz));
  RooFormulaVar gamma_J_sig_dz_d0bar("gamma_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_sig_dz, A_gamma_J_sig_dz));

  RooRealVar delta_J_sig_dz("#delta_{J_sig_dz}", "delta",1.95358e+00,0,5);
  RooRealVar A_delta_J_sig_dz("A_{delta_J_sig_dz}","",0);
  RooFormulaVar delta_J_sig_dz_d0("delta_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(delta_J_sig_dz, A_delta_J_sig_dz));
  RooFormulaVar delta_J_sig_dz_d0bar("delta_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(delta_J_sig_dz, A_delta_J_sig_dz));

  myRooJohnsonSU johnson_sig_dz_d0("johnson_sig_dz_d0", "johnson_sig_dz_d0", mKK, mu_J_sig_dz_d0, sigma_J_sig_dz_d0, delta_J_sig_dz_d0, gamma_J_sig_dz_d0);
  myRooJohnsonSU johnson_sig_dz_d0bar("johnson_sig_dz_d0bar", "johnson_sig_dz_d0bar", mKK, mu_J_sig_dz_d0bar, sigma_J_sig_dz_d0bar, delta_J_sig_dz_d0bar, gamma_J_sig_dz_d0bar);

  RooRealVar f_G1_sig_dz("f_{G1}_sig_dz", "relative_frac_g",1.78465e-01);
  RooRealVar A_f_G1_sig_dz("A_{f_G1_sig_dz}","",0);
  RooFormulaVar f_G1_sig_dz_d0("f_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(f_G1_sig_dz, A_f_G1_sig_dz));
  RooFormulaVar f_G1_sig_dz_d0bar("f_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(f_G1_sig_dz, A_f_G1_sig_dz));

  RooRealVar mu_G1_sig_dz("#mu_{G1}_sig_dz", "mean_gauss1",1.83931e+00);
  RooRealVar A_mu_G1_sig_dz("A_{mu_G1_sig_dz}","",0);
  RooFormulaVar mu_G1_sig_dz_d0("mu_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(mu_G1_sig_dz, A_mu_G1_sig_dz));
  RooFormulaVar mu_G1_sig_dz_d0bar("mu_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(mu_G1_sig_dz, A_mu_G1_sig_dz));

  RooRealVar sigma_G1_sig_dz("#sigma_{G1}_sig_dz", "sigma_gauss1",7.54561e-03);
  RooRealVar A_sigma_G1_sig_dz("A_{sigma_G1_sig_dz}","",0);
  RooFormulaVar sigma_G1_sig_dz_d0("sigma_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_G1_sig_dz, A_sigma_G1_sig_dz));
  RooFormulaVar sigma_G1_sig_dz_d0bar("sigma_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_G1_sig_dz, A_sigma_G1_sig_dz));

  RooRealVar sigma_G2_sig_dz("#sigma_{G2}_sig_dz", "sigma_gauss2",4.47656e-02);
  RooRealVar A_sigma_G2_sig_dz("A_{sigma_G2_sig_dz}","",0);
  RooFormulaVar sigma_G2_sig_dz_d0("sigma_G2_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_G2_sig_dz, A_sigma_G2_sig_dz));
  RooFormulaVar sigma_G2_sig_dz_d0bar("sigma_G2_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_G2_sig_dz, A_sigma_G2_sig_dz));

  RooGaussian gauss1_sig_dz_d0("gauss1_sig_dz_d0", "1st gaussian PDF", mKK, mu_G1_sig_dz_d0, sigma_G1_sig_dz_d0);
  RooGaussian gauss1_sig_dz_d0bar("gauss1_sig_dz_d0bar", "1st gaussian PDF", mKK, mu_G1_sig_dz_d0bar, sigma_G1_sig_dz_d0bar);

  RooGaussian gauss2_sig_dz_d0("gauss2_sig_dz_d0", "2nd gaussian PDF", mKK, mu_G1_sig_dz_d0, sigma_G2_sig_dz_d0);
  RooGaussian gauss2_sig_dz_d0bar("gauss2_sig_dz_d0bar", "2nd gaussian PDF", mKK, mu_G1_sig_dz_d0bar, sigma_G2_sig_dz_d0bar);

  RooAddPdf doublegaussian_sig_dz_d0("doublegaussian_sig_dz_d0", "doublegaussian_sig_dz_d0", gauss1_sig_dz_d0, gauss2_sig_dz_d0, f_G1_sig_dz_d0);
  RooAddPdf doublegaussian_sig_dz_d0bar("doublegaussian_sig_dz_d0bar", "doublegaussian_sig_dz_d0bar", gauss1_sig_dz_d0bar, gauss2_sig_dz_d0bar, f_G1_sig_dz_d0bar);

  RooAddPdf sig_mKK_d0("sig_mKK_d0","sig_mKK_d0", johnson_sig_dz_d0, doublegaussian_sig_dz_d0, f_J_sig_dz_d0);
  RooAddPdf sig_mKK_d0bar("sig_mKK_d0bar","sig_mKK_d0bar", johnson_sig_dz_d0bar, doublegaussian_sig_dz_d0bar, f_J_sig_dz_d0bar);

  //signal peak
  RooProdPdf signal_peak_d0("signal_peak_d0", "signal_peak_d0", sig_mKK_d0, sig_mD0pi_d0);
  RooProdPdf signal_peak_d0bar("signal_peak_d0bar", "signal_peak_d0bar", sig_mKK_d0bar, sig_mD0pi_d0bar);

  //random pion (common for all)
  RooRealVar m0("m0", "no data beyond this", 2.00441);
  RooRealVar kappa("C", "C", 2.23589e-03,-1e6,1e6);
  RooRealVar alpha("A", "A", 1.93367e+01,0,1e6);
  RooRealVar beta("B",  "B" ,0);
  RooDstD0BG bkg_mD0pi("bkg_mD0pi","bkg_mD0pi", mD0pi, m0, kappa, alpha, beta);

  //signal & random pion
  RooProdPdf signal_rnd_d0("signal_rnd_d0", "signal_rnd_d0", sig_mKK_d0, bkg_mD0pi);
  RooProdPdf signal_rnd_d0bar("signal_rnd_d0bar", "signal_rnd_d0bar", sig_mKK_d0bar, bkg_mD0pi);

  /*-----------------------K Pi-------------------------*/

  //kk
  RooRealVar f1_d0kpi("f_{G_kpi}", "relative fraction gaussian",3.09861e-01,0,1);
  RooRealVar A_f1_d0kpi("A_{f1_d0kpi}","",0);
  RooFormulaVar f1_d0kpi_d0("f1_d0kpi_d0","","(@0*(1 + @1))",RooArgList(f1_d0kpi, A_f1_d0kpi));
  RooFormulaVar f1_d0kpi_d0bar("f1_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(f1_d0kpi, A_f1_d0kpi));

  RooRealVar mu_G_d0kpi("#mu_{G_kpi}","mean_gaus",1.94463e+00);
  RooRealVar A_mu_G_d0kpi("A_{mu_G_d0kpi}","",0);
  RooFormulaVar mu_G_d0kpi_d0("mu_G_d0kpi_d0","","(@0*(1 + @1))",RooArgList(mu_G_d0kpi, A_mu_G_d0kpi));
  RooFormulaVar mu_G_d0kpi_d0bar("mu_G_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(mu_G_d0kpi, A_mu_G_d0kpi));

  RooRealVar sigma_G_d0kpi("#sigma_{G_kpi}", "sigma_gaus",9.39197e-03);
  RooRealVar A_sigma_G_d0kpi("A_{sigma_G_d0kpi}","",0);
  RooFormulaVar sigma_G_d0kpi_d0("sigma_G_d0kpi_d0","","(@0*(1 + @1))",RooArgList(sigma_G_d0kpi, A_sigma_G_d0kpi));
  RooFormulaVar sigma_G_d0kpi_d0bar("sigma_G_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G_d0kpi, A_sigma_G_d0kpi));

  RooGaussian gauss_d0kpi_d0("gauss_d0kpi_d0", "gauss component", mKK, mu_G_d0kpi_d0, sigma_G_d0kpi_d0);
  RooGaussian gauss_d0kpi_d0bar("gauss_d0kpi_d0bar", "gauss component", mKK, mu_G_d0kpi_d0bar, sigma_G_d0kpi_d0bar);

  RooRealVar gamma_J_d0kpi("#gamma_{J_kpi}", "gamma",-1.56041e+00,-5,5);
  RooRealVar A_gamma_J_d0kpi("A_{gamma_J_d0kpi}","",0);
  RooFormulaVar gamma_J_d0kpi_d0("gamma_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(gamma_J_d0kpi, A_gamma_J_d0kpi));
  RooFormulaVar gamma_J_d0kpi_d0bar("gamma_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_d0kpi, A_gamma_J_d0kpi));

  RooRealVar delta_J_d0kpi("#delta_{J_kpi}", "delta",4.04397e-01,0,5);
  RooRealVar A_delta_J_d0kpi("A_{delta_J_d0kpi}","",0);
  RooFormulaVar delta_J_d0kpi_d0("delta_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(delta_J_d0kpi, A_delta_J_d0kpi));
  RooFormulaVar delta_J_d0kpi_d0bar("delta_K_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(delta_J_d0kpi, A_delta_J_d0kpi));

  RooRealVar mu_J_d0kpi("#mu_{J_kpi}", "mean",1.96138e+00,1.94,1.98);
  RooRealVar A_mu_J_d0kpi("A_{mu_J_d0kpi}","",0);
  RooFormulaVar mu_J_d0kpi_d0("mu_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(mu_J_d0kpi, A_mu_J_d0kpi));
  RooFormulaVar mu_J_d0kpi_d0bar("mu_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(mu_J_d0kpi, A_mu_J_d0kpi));

  RooRealVar sigma_J_d0kpi("#sigma_{J}_d0kpi", "sigma",1.19344e-02);
  RooRealVar A_sigma_J_d0kpi("A_{sigma_J_d0kpi}","",0);
  RooFormulaVar sigma_J_d0kpi_d0("sigma_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(sigma_J_d0kpi, A_sigma_J_d0kpi));
  RooFormulaVar sigma_J_d0kpi_d0bar("sigma_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_d0kpi, A_sigma_J_d0kpi));

  myRooJohnsonSU johnson_d0kpi_d0("johnson_d0kpi_d0", "Johnson PDF", mKK, mu_J_d0kpi_d0, sigma_J_d0kpi_d0, delta_J_d0kpi_d0, gamma_J_d0kpi_d0);
  myRooJohnsonSU johnson_d0kpi_d0bar("johnson_d0kpi_d0bar", "Johnson PDF", mKK, mu_J_d0kpi_d0bar, sigma_J_d0kpi_d0bar, delta_J_d0kpi_d0bar, gamma_J_d0kpi_d0bar);

  RooAddPdf kpi_mKK_d0("kpi_mKK_d0","kpi_mKK_d0",gauss_d0kpi_d0,johnson_d0kpi_d0,f1_d0kpi_d0);
  RooAddPdf kpi_mKK_d0bar("kpi_mKK_d0bar","kpi_mKK_d0bar",gauss_d0kpi_d0bar,johnson_d0kpi_d0bar,f1_d0kpi_d0bar);

  //kpi peaking background
  RooProdPdf kpi_peak_d0("kpi_peak_d0", "kpi_peak_d0", kpi_mKK_d0, sig_mD0pi_d0);//as the shape of m(d0pi) is same as signal
  RooProdPdf kpi_peak_d0bar("kpi_peak_d0bar", "kpi_peak_d0bar", kpi_mKK_d0bar, sig_mD0pi_d0bar);//as the shape of m(d0pi) is same as signal

  //kpi & random pion
  RooProdPdf kpi_rnd_d0("kpi_rnd_d0", "kpi_rnd_d0", kpi_mKK_d0, bkg_mD0pi);
  RooProdPdf kpi_rnd_d0bar("kpi_rnd_d0bar", "kpi_rnd_d0bar", kpi_mKK_d0bar, bkg_mD0pi);

  /*-----------------------M U L T I B O D Y-------------------------*/
  RooRealVar f_J1_mult("f_{J1_mult}", "",2.30817e-01);
  RooRealVar A_f_J1_mult("A_{f_J1_mult}","",0);
  RooFormulaVar f_J1_mult_d0("f_J1_mult_d0","","(@0*(1 + @1))",RooArgList(f_J1_mult,  A_f_J1_mult));
  RooFormulaVar f_J1_mult_d0bar("f_J1_mult_d0","","(@0*(1- @1))",RooArgList(f_J1_mult, A_f_J1_mult));

  RooRealVar mu_J1_mult("#mu_{J1_mult}","mean_johnson",2.00641e+00);
  RooRealVar A_mu_J1_mult("A_{mu_J1_mult}","",0);
  RooFormulaVar mu_J1_mult_d0("mu_J1_mult_d0","","(@0*(1 + @1))",RooArgList(mu_J1_mult, A_mu_J1_mult));
  RooFormulaVar mu_J1_mult_d0bar("mu_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(mu_J1_mult, A_mu_J1_mult));

  RooRealVar lambda_J1_mult("#lambda_{J1_mult}", "sigma_johnson",9.58128e-04);
  RooRealVar A_lambda_J1_mult("A_{lambda_J1_mult}","",0);
  RooFormulaVar lambda_J1_mult_d0("lambda_J1_mult_d0","","(@0*(1 + @1))",RooArgList(lambda_J1_mult, A_lambda_J1_mult));
  RooFormulaVar lambda_J1_mult_d0bar("lambda_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(lambda_J1_mult, A_lambda_J1_mult));

  RooRealVar gamma_J1_mult("#gamma_{J1_mult}", "gamma",-3.23022e+00);
  RooRealVar A_gamma_J1_mult("A_{gamma_J1_mult}","",0);
  RooFormulaVar gamma_J1_mult_d0("gamma_J1_mult_d0","","(@0*(1 + @1))",RooArgList(gamma_J1_mult, A_gamma_J1_mult));
  RooFormulaVar gamma_J1_mult_d0bar("gamma_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(gamma_J1_mult, A_gamma_J1_mult));

  RooRealVar delta_J1_mult("#delta_{J1_mult}", "delta",1.43030e-01);
  RooRealVar A_delta_J1_mult("A_{delta_J1_mult}","",0);
  RooFormulaVar delta_J1_mult_d0("delta_J1_mult_d0","","(@0*(1 + @1))",RooArgList(delta_J1_mult, A_delta_J1_mult));
  RooFormulaVar delta_J1_mult_d0bar("delta_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(delta_J1_mult, A_delta_J1_mult));

  myRooJohnsonSU johnson1_mult_d0("johnson1_d0", "Johnson PDF", mD0pi, mu_J1_mult_d0, lambda_J1_mult_d0, delta_J1_mult_d0, gamma_J1_mult_d0);
  myRooJohnsonSU johnson1_mult_d0bar("johnson1_d0bar", "Johnson PDF", mD0pi, mu_J1_mult_d0bar, lambda_J1_mult_d0bar, delta_J1_mult_d0bar, gamma_J1_mult_d0bar);

  RooRealVar mu_J2_mult("#mu_{J2_mult}","mean_johnson",2.00524e+00);
  RooRealVar A_mu_J2_mult("A_{mu_J2_mult}","",0);
  RooFormulaVar mu_J2_mult_d0("mu_J2_mult_d0","","(@0*(1 + @1))",RooArgList(mu_J2_mult, A_mu_J2_mult));
  RooFormulaVar mu_J2_mult_d0bar("mu_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(mu_J2_mult, A_mu_J2_mult));

  RooRealVar lambda_J2_mult("#lambda_{J2_mult}", "sigma_johnson",3.24842e-03 );
  RooRealVar A_lambda_J2_mult("A_{lambda_J2_mult}","",0);
  RooFormulaVar lambda_J2_mult_d0("lambda_J2_mult_d0","","(@0*(1 + @1))",RooArgList(lambda_J2_mult, A_lambda_J2_mult));
  RooFormulaVar lambda_J2_mult_d0bar("lambda_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(lambda_J2_mult, A_lambda_J2_mult));

  RooRealVar delta_J2_mult("#delta_{J2_mult}", "delta",2.30106e+00);
  RooRealVar A_delta_J2_mult("A_{delta_J2_mult}","",0);
  RooFormulaVar delta_J2_mult_d0("delta_J2_mult_d0","","(@0*(1 + @1))",RooArgList(delta_J2_mult, A_delta_J2_mult));
  RooFormulaVar delta_J2_mult_d0bar("delta_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(delta_J2_mult, A_delta_J2_mult));

  myRooJohnsonSU johnson2_mult_d0("johnson2_d0", "Johnson PDF", mD0pi, mu_J2_mult_d0, lambda_J2_mult_d0, delta_J2_mult_d0, gamma_J1_mult_d0);
  myRooJohnsonSU johnson2_mult_d0bar("johnson2_d0bar", "Johnson PDF", mD0pi, mu_J2_mult_d0bar, lambda_J2_mult_d0bar, delta_J2_mult_d0bar, gamma_J1_mult_d0bar);

  RooAddPdf mult_mD0pi_d0("mult_mD0pi_d0", "mult_mD0pi_d0", johnson1_mult_d0, johnson2_mult_d0, f_J1_mult_d0);
  RooAddPdf mult_mD0pi_d0bar("mult_mD0pi_d0bar", "mult_mD0pi_d0bar", johnson1_mult_d0bar, johnson2_mult_d0bar, f_J1_mult_d0bar);
  // myRooJohnsonSU mult_mD0pi_d0("mult_mD0pi_d0","mult_mD0pi_d0", mD0pi, mu_J_mult_d0, sigma_J_mult_d0, delta_J_mult_d0, gamma_J_mult_d0);
  // myRooJohnsonSU mult_mD0pi_d0bar("mult_mD0pi_d0bar","mult_mD0pi_d0bar", mD0pi, mu_J_mult_d0bar, sigma_J_mult_d0bar, delta_J_mult_d0bar, gamma_J_mult_d0bar);

  //kk
  RooRealVar a1_mult("a_{1_mult}", "A1", -1.33978e+01);
  RooRealVar A_a1_mult("A_{a1_mult}","",0);
  RooFormulaVar a1_mult_d0("a1_mult_d0","","(@0*(1 + @1))",RooArgList(a1_mult, A_a1_mult));
  RooFormulaVar a1_mult_d0bar("a1_mult_d0bar","","(@0*(1- @1))",RooArgList(a1_mult, A_a1_mult));

  RooExponential mult_mKK_d0("mult_mKK_d0", "mult_mKK_d0", mKK, a1_mult_d0);
  RooExponential mult_mKK_d0bar("mult_mKK_d0bar", "mult_mKK_d0bar", mKK, a1_mult_d0bar);

  RooProdPdf mult_peak_d0("mult_peak_d0", "mult_peak_d0", mult_mKK_d0, mult_mD0pi_d0);
  RooProdPdf mult_peak_d0bar("mult_peak_d0bar", "mult_peak_d0bar", mult_mKK_d0bar, mult_mD0pi_d0bar);

  //multibody & random pion
  RooProdPdf mult_rnd_d0("mult_rnd_d0","mult_rnd_d0", mult_mKK_d0, bkg_mD0pi);
  RooProdPdf mult_rnd_d0bar("mult_rnd_d0bar","mult_rnd_d0bar", mult_mKK_d0bar, bkg_mD0pi);

  /*----------------------- Ds+ -> K+ K- pi+ -------------------------*/
  //d0pi
  RooRealVar slope_ds("slope_ds", "Slope1 of Polynomial",7.55528e-01);
  RooRealVar A_slope_ds("A_{slope_ds}","",0);
  RooFormulaVar slope_ds_d0("slope_ds_d0","","(@0*(1 + @1))",RooArgList(slope_ds, A_slope_ds));
  RooFormulaVar slope_ds_d0bar("slope_ds_d0bar","","(@0*(1- @1))",RooArgList(slope_ds, A_slope_ds));

  RooChebychev ds_mD0pi_d0("ds_mD0pi_d0","Chebshev Polynomial", mD0pi,RooArgList(slope_ds_d0));
  RooChebychev ds_mD0pi_d0bar("ds_mD0pi_d0bar","Chebshev Polynomial", mD0pi,RooArgList(slope_ds_d0bar));

  //kk
  RooRealVar mu_0("#mu_{0}", "",7.06528e-06);
  RooRealVar A_mu_0("A_{mu_0}","",0);
  RooFormulaVar mu_0_d0("mu_0_d0","","(@0*(1 + @1))",RooArgList(mu_0, A_mu_0));
  RooFormulaVar mu_0_d0bar("mu_0_d0bar","","(@0*(1- @1))",RooArgList(mu_0, A_mu_0));

  RooRealVar constant("constant", "",3.83319);
  RooFormulaVar mu_ds_d0("#mu_{ds_d0}", "@0+@1-@2",RooArgList(mu_0_d0,constant, mD0pi));
  RooFormulaVar mu_ds_d0bar("#mu_{ds_d0bar}", "@0+@1-@2",RooArgList(mu_0_d0bar,constant, mD0pi));

  RooRealVar sigma_ds("#sigma_{ds}", "",2.88912e-03);
  RooRealVar A_sigma_ds("A_{sigma_ds}","",0);
  RooFormulaVar sigma_ds_d0("sigma_ds_d0","","(@0*(1 + @1))",RooArgList(sigma_ds, A_sigma_ds));
  RooFormulaVar sigma_ds_d0bar("sigma_ds_d0bar","","(@0*(1- @1))",RooArgList(sigma_ds, A_sigma_ds));

  RooRealVar gamma_ds("#gamma_{ds}", "",0);
  RooRealVar A_gamma_ds("A_{gamma_ds}","",0);
  RooFormulaVar gamma_ds_d0("gamma_ds_d0","","(@0*(1 + @1))",RooArgList(gamma_ds, A_gamma_ds));
  RooFormulaVar gamma_ds_d0bar("gamma_ds_d0bar","","(@0*(1- @1))",RooArgList(gamma_ds, A_gamma_ds));

  RooRealVar delta_ds("#delta_{ds}", "",6.47505e-01);
  RooRealVar A_delta_ds("A_{delta_ds}","",0);
  RooFormulaVar delta_ds_d0("delta_ds_d0","","(@0*(1 + @1))",RooArgList(delta_ds, A_delta_ds));
  RooFormulaVar delta_ds_d0bar("delta_ds_d0bar","","(@0*(1- @1))",RooArgList(delta_ds, A_delta_ds));

  myRooJohnsonSU ds_mKK_d0("ds_mKK_d0","johnson_d0",mKK, mu_ds_d0,sigma_ds_d0,delta_ds_d0,gamma_ds_d0);
  myRooJohnsonSU ds_mKK_d0bar("ds_mKK_d0bar","johnson_d0bar",mKK, mu_ds_d0bar,sigma_ds_d0bar,delta_ds_d0bar,gamma_ds_d0bar);

  RooProdPdf ds_peak_d0("ds_peak_d0", "ds_peak_d0", ds_mD0pi_d0, Conditional(ds_mKK_d0, mKK));
  RooProdPdf ds_peak_d0bar("ds_peak_d0bar", "ds_peak_d0bar", ds_mD0pi_d0bar, Conditional(ds_mKK_d0bar, mKK));

  /*------------------------C O M B I N A T O R I A L----------------------------*/

  RooRealVar c("c_{comb}", "Slope1 of Polynomial",-2.97793e-01, -1,1);
  RooRealVar A_c("A_{c}","",0);
  RooFormulaVar c_d0("c_d0","","(@0*(1 + @1))",RooArgList(c, A_c));
  RooFormulaVar c_d0bar("c_d0bar","","(@0*(1- @1))",RooArgList(c, A_c));

  RooChebychev comb_mKK_d0("chebpol_d0","Chebshev Polynomial", mKK,RooArgList(c_d0));
  RooChebychev comb_mKK_d0bar("chebpol_d0bar","Chebshev Polynomial", mKK,RooArgList(c_d0bar));

  RooProdPdf comb_bkg_d0("comb_bkg_d0", "comb_bkg_d0", comb_mKK_d0, bkg_mD0pi);
  RooProdPdf comb_bkg_d0bar("comb_bkg_d0bar", "comb_bkg_d0bar", comb_mKK_d0bar, bkg_mD0pi);

  /******   GENERATE TOY MC   *******/

  TFile *file = TFile::Open("weight_toy_study.root");
  TH1F *h_weights = (TH1F*) file->Get("weight_histo");

  //total number of events to be generated
  const int N = 1077935;

  //component fractions
  double f_sig = 0.286;
  double f_sig_rnd = 0.03;
  double f_kpi = 0.223;
  double f_kpi_rnd = 0.017;
  double f_mult = 0.165;
  double f_mult_rnd = 0.02;
  double f_ds = 0.016;
  double f_comb = 0.243;

  //component asymmetries
  double a_sig = 0.17;
  double a_sig_rnd = 1.30;
  double a_kpi = -2.16;
  double a_kpi_rnd = -4.7;
  double a_mult = -0.77;
  double a_mult_rnd = 6.93;
  double a_ds = 4.6;
  double a_comb = 0.76;

  //objects to temporarily hold generated variables
  int pisoft_charge(0);
  RooDataSet *tmp;

  //initialize random number generator
  auto r = new TRandom3();

  //total numer of toys to be generated
  // int n_toys(2);

  //loop over toys
  // for (int i_toy=1; i_toy<=n_toys; ++i_toy) {

    //set seed for random number generator
    r->SetSeed(i_toy);

    //sample total number of events to be generated for this toy
    int n = r->Poisson(N);

    //create output file
    std::ofstream fout(Form("toy_%d.txt",i_toy));

    //loop over events to be generated
    for (int i=0; i<n; ++i) {

      //decide which component to sample
      double f = r->Uniform(0,1);

      if (f < f_sig) {//sample a signal candidate

        //decide candidate's flavor
        if (r->Uniform(0,1) < (1 + a_sig)/2.) {//sample a D0 candidate

          //set soft pion's charge
          pisoft_charge = +1;

          //sample fit variables
          tmp = signal_peak_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {//sample a D0bar candidate
          pisoft_charge = -1;
          tmp = signal_peak_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      } else if (f < (f_sig + f_sig_rnd)) {//sample signal_rnd
      
        if (r->Uniform(0,1) < (1 + a_sig_rnd)/2. ) {
          pisoft_charge = +1;
          tmp = signal_rnd_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = signal_rnd_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      } else if (f < (f_sig + f_sig_rnd + f_kpi)) {//sample kpi
      
        if (r->Uniform(0,1) < (1 + a_kpi)/2. ) {
          pisoft_charge = +1;
          tmp = signal_peak_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = signal_peak_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      }else if (f < (f_sig + f_sig_rnd + f_kpi + f_kpi_rnd)) {//sample kpi_rnd
      
        if (r->Uniform(0,1) < (1 + a_kpi_rnd)/2.) {
          pisoft_charge = +1;
          tmp = kpi_rnd_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = kpi_rnd_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      }else if (f < (f_sig + f_sig_rnd + f_kpi + f_kpi_rnd + f_mult)) {//sample multibody
      
        if (r->Uniform(0,1) < (1 + a_mult)/2.) {
          pisoft_charge = +1;
          tmp = mult_peak_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = mult_peak_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      }else if (f < (f_sig + f_sig_rnd + f_kpi + f_kpi_rnd + f_mult + f_mult_rnd)) {//sample multibody_rnd
      
        if (r->Uniform(0,1) < (1 + a_mult_rnd)/2.) {
          pisoft_charge = +1;
          tmp = mult_rnd_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = mult_rnd_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      }else if (f < (f_sig + f_sig_rnd + f_kpi + f_kpi_rnd + f_mult + f_mult_rnd + f_comb)){//sample comb
      //else (f < (f_sig + f_sig_rnd + f_kpi + f_kpi_rnd + f_mult + f_mult_rnd + f_comb)){//sample comb
      
        if (r->Uniform(0,1) < (1 + a_comb)/2.) {
          pisoft_charge = +1;
          tmp = comb_bkg_d0.generate(RooArgSet(mKK,mD0pi),1);

        } else {
          pisoft_charge = -1;
          tmp = comb_bkg_d0bar.generate(RooArgSet(mKK,mD0pi),1);
        }  
      }

      //write out
      fout << tmp->get(0)->getRealValue("mKK") << "\t";
      fout << tmp->get(0)->getRealValue("mD0pi") << "\t";

      // for (const auto next : tmp->get(0)->get()) fout << *next << '\t';
      fout << pisoft_charge << '\t' << h_weights->GetRandom() << std::endl;
    }
    fout.close();
  }
 // }
